﻿using Landsbankinn;
using System;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace Landsbankinn.B2B.Sample.B2B
{
    class LBB2BClient
    {
        private CookieContainer cookies = new CookieContainer();

        public string UserName { get; set; }

        public string Password { get; set; }

        public string URL { get; set; }

        public string SessionId { get; private set; }

        public void Login()
        {
            SessionId = null;

            LI_Innskra login = new LI_Innskra();
            login.notandanafn = UserName;
            login.lykilord = Password;
            login.version = 1.1m;

            LI_Innskra_svar response = (LI_Innskra_svar)SendAndReceive(login, Type.GetType("Landsbankinn.LI_Innskra_svar"));
            SessionId = response.seta;
        }

        public void Logout()
        {
            if (string.IsNullOrEmpty(URL))
                return;

            if (string.IsNullOrEmpty(SessionId))
                return;
            
            LI_Utskra logout = new LI_Utskra();
            logout.version = 1.1m;
            logout.seta = SessionId;

            SendAndReceive(logout, Type.GetType("Landsbankinn.LI_Utskra_svar"));

            SessionId = null;
        }

        public object SendAndReceive(object oOutObj, Type typeResponse)
        {
            // Serialize output object to memory
            MemoryStream memoryStream = new MemoryStream();
            XmlTextWriter memoryWriter = new XmlTextWriter(memoryStream, Encoding.UTF8);
            XmlSerializer serializer = new XmlSerializer(oOutObj.GetType());
            serializer.Serialize(memoryWriter, oOutObj);

            // Prepare document to send
            XmlDocument doc = new XmlDocument();
            doc.PreserveWhitespace = true;
            memoryStream.Position = 0;
            doc.Load(memoryStream);
            memoryStream.Close();

            if (SessionId != null)
                ReplaceSessionId(doc);

            // Send document
            HttpWebRequest request = CreateRequest();
            Stream requestStream = request.GetRequestStream();
            XmlTextWriter serviceWriter = new XmlTextWriter(requestStream, Encoding.UTF8);
            serviceWriter.Formatting = Formatting.Indented;
            doc.WriteTo(serviceWriter);
            serviceWriter.Close();
            requestStream.Close();

            HttpWebResponse response = (HttpWebResponse)request.GetResponse();
            if (response.StatusCode != HttpStatusCode.OK)
            {
                throw new Exception($"{response.StatusCode} - {response.StatusDescription}");
            }

            return Deserialize(response.GetResponseStream(), typeResponse);
        }

        private object Deserialize(Stream stream, Type expectedType)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(stream);

            XmlNodeReader reader = new XmlNodeReader(doc.DocumentElement);

            if (doc.DocumentElement.Name == "LI_Villa")
            {
                XmlSerializer xsError = new XmlSerializer(Type.GetType("Landsbankinn.LI_Villa"));
                LI_Villa error = (LI_Villa)xsError.Deserialize(reader);
                throw new Exception($"{error.villa} - {error.villubod}");
            }

            XmlSerializer serializer = new XmlSerializer(expectedType);
            return serializer.Deserialize(reader);
        }

        private bool ReplaceSessionId(XmlDocument xmlDoc, string nodeName)
        {
            XmlNode node = xmlDoc.DocumentElement.SelectSingleNode(nodeName);

            if (node == null)
                return false;

            node.InnerText = SessionId;
            return true;
        }

        private void ReplaceSessionId(XmlDocument xmlDoc)
        {
            if (!ReplaceSessionId(xmlDoc, "seta"))
                ReplaceSessionId(xmlDoc, "session_id");
        }

        private HttpWebRequest CreateRequest()
        {
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(URL);
            request.Method = "POST";
            request.ContentType = "text/xml";
            request.UserAgent = "Landsbankinn B2B Sample Client 1.0";
            request.CookieContainer = cookies;

            return request;
        }

        public void DisplayCurrencyRates(DateTime date)
        {
            try
            {
                LI_Fyrirspurn_gengi_gjaldmidla request = new LI_Fyrirspurn_gengi_gjaldmidla();
                request.version = 1.1m;
                request.seta = "";
                request.dags = date;
                request.gengistegund = LI_gengistegund_type.A;

                LI_Fyrirspurn_gengi_gjaldmidla_svar response =
                    (LI_Fyrirspurn_gengi_gjaldmidla_svar)SendAndReceive(request, Type.GetType("Landsbankinn.LI_Fyrirspurn_gengi_gjaldmidla_svar"));

                var currencyRates = response.gjaldmidlar;

                Console.WriteLine(currencyRates.Where(x => x.iso_takn == LI_ISO_takn_gjaldmidils_type.SEK).Select(x => $"{x.iso_takn} {x.kaupgengi} - {x.solugengi}").FirstOrDefault());
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }
    }
}
