﻿using System;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;
using System.Text;

namespace Landsbankinn.B2B.Sample
{
    abstract class B2BIOBSClient
    {
        public string UserName { get; set; }

        public string Password { get; set; }

        public string URL { get; set; }

        public string Endpoint { get; set; }

        public string ServiceCertificateName { get; set; }

        public string ClientCertificateName { get; set; }
        
        /// <summary>
        /// Creating WCF bindings 
        /// </summary>
        /// <returns></returns>
        protected CustomBinding CreateBinding()
        {
            MessageSecurityVersion secProfile = MessageSecurityVersion.WSSecurity10WSTrustFebruary2005WSSecureConversationFebruary2005WSSecurityPolicy11BasicSecurityProfile10;

            AsymmetricSecurityBindingElement sec = (AsymmetricSecurityBindingElement)SecurityBindingElement.CreateMutualCertificateBindingElement(secProfile);
            sec.EndpointSupportingTokenParameters.Signed.Add(new UserNameSecurityTokenParameters());
            sec.MessageSecurityVersion = secProfile;
            sec.AllowSerializedSigningTokenOnReply = true;

            sec.IncludeTimestamp = true;
            sec.MessageProtectionOrder = MessageProtectionOrder.SignBeforeEncrypt;
            sec.EnableUnsecuredResponse = true;

            CustomBinding binding = new CustomBinding();
            binding.Elements.Add(sec);
            binding.Elements.Add(new TextMessageEncodingBindingElement(MessageVersion.Soap11, Encoding.UTF8));

            var uri = new Uri(this.URL);
            BindingElement transportBindingElement;
            if (uri.Scheme == "https")
            {
                transportBindingElement = new HttpsTransportBindingElement
                {
                    MaxReceivedMessageSize = 2147483647, // The maximum size, in bytes, for a message that is processed by the binding. The default value is 65,536 bytes.
                };
            }
            else
            {
                transportBindingElement = new HttpTransportBindingElement
                {
                    MaxReceivedMessageSize = 2147483647,
                };
            }

            binding.Elements.Add(transportBindingElement);
            return binding;
        }

        protected EndpointAddress CreateEndpointAddress(string endpoint)
        {
            return new EndpointAddress(
                       new Uri(URL + endpoint),
                       new DnsEndpointIdentity("Landsbankinn hf."), // must be same as CN of server certificate
                       new AddressHeaderCollection()
                   );
        }

        protected void SetupClientSecurity(dynamic client)
        {
            client.ClientCredentials.UserName.UserName = UserName;
            client.ClientCredentials.UserName.Password = Password;

            // Set client certificate
            client.ClientCredentials.ClientCertificate.SetCertificate(
                StoreLocation.LocalMachine,
                StoreName.My,
                X509FindType.FindBySubjectDistinguishedName,
                ClientCertificateName);

            // Set service certificate
            client.ClientCredentials.ServiceCertificate.SetDefaultCertificate(
                StoreLocation.LocalMachine,
                StoreName.TrustedPeople,
                X509FindType.FindBySubjectDistinguishedName,
                ServiceCertificateName);

            client.Endpoint.Contract.ProtectionLevel = System.Net.Security.ProtectionLevel.Sign;
            client.ClientCredentials.ServiceCertificate.Authentication.CertificateValidationMode = X509CertificateValidationMode.None;
        }
    }
}
