﻿using System;
using System.Configuration;

namespace Landsbankinn.B2B.Sample
{
    class Program
    {
        static void Main(string[] args)
        {
            /********************************
              In this sample solution you can find examples that know how to communicate with all the different
              B2B schemas Landsbankinn offers its customers. This example solution was built in 2020 and does
              not use the WSE 3.0 framework to connect to the IOBS 2005 B2B endpoints. It replaces two old WinForms 
              example clients which were previous published on landsbankinn.is.

              This project is a good start if you are developing your own systems that talk to B2B. There are three
              different B2B schemas and here we provide examples for each one.

              To know more about the B2B offerings and the different methods available please see the B2B pages
              on https://www.landsbankinn.is/
            ********************************/

            string username = ConfigurationManager.AppSettings["Username"];
            string password = ConfigurationManager.AppSettings["Password"];
            string serverUri = ConfigurationManager.AppSettings["ServerUri"];
            string serviceCertificateName = ConfigurationManager.AppSettings["ServiceCertificateName"];
            string clientCertificateName = ConfigurationManager.AppSettings["ClientCertificateName"];

            DateTime date = DateTime.Parse("2020-01-03");



            /********************************
              Example of calling endpoints on the Landsbankinn B2B schema. In Landsbankinn schema users first 
              have to do login and preferably keep track of the session id and reuse it. The session id has a 
              timeout of about 15 minutes. If possible a logout to end the B2B session when client processing
              has finished interacting with B2B is appreciated. Please note that opening multiple sessions
              without closing them will result in the client being tarpitted. 
            ********************************/

            var b2bClient = new B2B.LBB2BClient
            {
                UserName = username,
                Password = password,
                URL = serverUri + "process.ashx"
            };

            // Login to get a session id (our B2BClient class stores the session id)
            b2bClient.Login();

            // Get currency exchange rates
            Console.WriteLine($"\nB2B: Getting currency exchange rates for {date:dd.MM.yyyy}");
            b2bClient.DisplayCurrencyRates(date);
            
            // Being a nice B2B user and logout
            b2bClient.Logout();



            /********************************
              Example of calling endpoints on the IOBS 2005 (aka IOBS v1) schema. In this schema the user sends 
              the credentials and signs each message so no explicit login/logout.
            ********************************/

            var iobs2005Client = new B2BIOBS2005.Client
            {
                UserName = username,
                Password = password,
                URL = serverUri,
                ServiceCertificateName = serviceCertificateName,
                ClientCertificateName = clientCertificateName
            };

            // Get currency exchange rates
            Console.WriteLine($"\nIOBS2005: Getting  currency exchange rates for {date:dd.MM.yyyy}");
            iobs2005Client.DisplayCurrencyRates(date);
            


            /********************************
              Example of calling endpoints on the IOBS 2013 (aka IOBS v2) schema. Like in IOBS 2005 there are no
              explicit login/logout methods.
            ********************************/

            var iobs2013Client = new B2BIOBS2013.Client
            {
                UserName = username,
                Password = password,
                URL = serverUri,
                ServiceCertificateName = serviceCertificateName,
                ClientCertificateName = clientCertificateName
            };

            // Get currency exchange rates
            Console.WriteLine($"\nIOBS2013: Getting currency exchange rates for {date:dd.MM.yyyy}");
            iobs2013Client.DisplayCurrencyRates(date);



            Console.WriteLine("Press any key to exit...");
            Console.ReadKey();
        }
    }
}
